#  tests for pywavelets-0.5.2-py36ha460e8f_0 (this is a generated file);
print('===== testing package: pywavelets-0.5.2-py36ha460e8f_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import pywt
pywt.test()
#  --- run_test.py (end) ---

print('===== pywavelets-0.5.2-py36ha460e8f_0 OK =====');
